/*  EX: Simple 1-level section editing for PmWiki
    Copyright 2021-2023 Petko Yotov https://www.pmwiki.org/support
    This file is part of PmWiki; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
    by the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.  See pmwiki.php for full details.
*/

var EXparts = [];
(function(){
  var log = console.log;
  var KeepToken = "\034\034", KPV = [];
  var restoreRX = new RegExp(KeepToken+'(\\d+)'+KeepToken, 'g');
  function Restore(all, n) { return KPV[parseInt(n)]; }
  function keep0(text) {
    if(text === '') return '';
    KPV.push(text.replace(restoreRX, Restore));
    return KeepToken+(KPV.length-1)+KeepToken;
  }
  function adjab(el, html) { el.insertAdjacentHTML('afterbegin', html); }
  
  function textblur() {
    var sv = selEX.value, tv = text.value;
    if(sv == -1) parseText();
    else {
      EXparts[sv] = tv;
      var o = selEX.options[selEX.selectedIndex];
      o.textContent = tv.replace(/[\s\n]+/g, ' ').trim().slice(0, 80);
    }
  }
  
  function selectchange() {
    var prev = parseInt(selEX.dataset.prev);
    var v = parseInt(selEX.value);
    var all = EXparts.join('\n').replace(/^\n+/, '');
    if(v>=0) {
      var last = EXparts[v];
      parseText(all);
      for(var i=0; i<EXparts.length; i++) {
        if(EXparts[i].indexOf(last)>=0) {
          text.value = EXparts[i];
          selEX.value = i;          
          selEX.dataset.prev = i;
          text.dispatchEvent(new Event('input'));
          return;
        }
      }
    }
    // may be fill page, or some error
    text.value = all;
    parseText();
    selEX.value = -1;
    selEX.dataset.prev = -1;
    text.dispatchEvent(new Event('input'));
  }
  
  function parseText(content) {
    if(!content) content = text.value;
    KPV = [];
    var rx = [
      /(?:\(:markup.*?:\)[\n\s]*)?\[([@=])[\s\S]*?\1\]/g, // Escaped, markup+[@
      /\(:(?:nl|groupheader):\)/g, // PTV
      /\(: *\w[\w-]* *:[^\)][\s\S]*?:\)/g, // PTV
      /\(:markup.*?:\)[\s\S]*?\(:markupend:\)/g,
      /\\\n/g // joined lines
    ];
    for(var i=0; i<rx.length; i++) {
      content = content.replace(rx[i], keep0);
    }
    var hlevels = content.match(/^!{1,6}/mg);
    if(!hlevels) {
      var sections = [text.value];
    }
    else {
      var maxheading = 6;
      for(var i=0; i<hlevels.length; i++) maxheading = Math.min(maxheading, hlevels[i].length);
      
      var hrx = new RegExp('((?:\n?\\[\\[\\#[-\\w]+\\]\\][\\s]*)?(?:\n|^)!{'+maxheading+'}(?!!)[^\n]*)', 'g');
      var sections = content.split(hrx);
      sections[0] = sections[0].replace(restoreRX, Restore);
      for(var i=1; i<sections.length; i++) {
        sections[i] = sections[i].replace(/^\n/, '').replace(restoreRX, Restore);
      }
    }
    while(selEX.options.length>2) selEX.removeChild(selEX.options[2]);
    
    
    var cnt = 0;
    EXparts = [ sections[0] ];
    if(sections.length>1)
      addOption(0, selEX.dataset.head);
      
    for(var i=1; i<sections.length-1; i+=2) {
      cnt++;
      var h = sections[i];
      var c = h +'\n'+ sections[i+1];
      EXparts.push(c);
      addOption(cnt, c);
    }
  }
  
  function addOption(value, text){
    var o = document.createElement('option');
    o.value = value;
    if(value === false) o.disabled = true;
    o.textContent = text.replace(/[\s\n]+/g, ' ').trim().replace(/^(.{80}).{3,}$/, '$1...');
    selEX.appendChild(o);
  }
  

  function initEX() {
    if(typeof text == 'undefined' || !text.form) return;

    var _script = document.querySelector('script[src*="/ex.js"][data-i18n]');
    if(!_script) return;
    var i18n = JSON.parse(_script.dataset.i18n);
    var s = '<select name="ex" id="selEX" data-prev="-1"></select> ';
    var gb = document.querySelector('.GUIButtons');
    if(gb) adjab(gb, s);
    else adjab(text.form, s);
    addOption(false, i18n.editsection);
    addOption(-1, i18n.fullpage);
    selEX.title = i18n.editsection;
    selEX.dataset.head = i18n.headsection;
    
    selEX.addEventListener('change', selectchange);
    text.addEventListener('blur', textblur);
   
    adjab(text.form, '<input type="hidden" name="EXtext" id="EXtext" />');
    
    parseText();
    if(typeof prevEX != 'undefined') {
      selEX.value = prevEX.value;
      selectchange();
    }                      
    text.form.addEventListener('submit', function(){
      EXtext.value = EXparts.join('\n').replace(/^\n+/, '');
    });
  }
    
  initEX();
})();



